/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.wtp.releng.tools.component.adopters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.eclipse.wtp.releng.tools.component.util.CommandOptionParser;
import org.eclipse.wtp.releng.tools.component.xsl.XSLUtil;

public class CombineClass2Reference
{
  private HashMap map;
  private Collection src;
  private String output;

  public Collection getSrc()
  {
    return src;
  }

  public void setSrc(Collection src)
  {
    this.src = src;
  }

  public String getOutput()
  {
    return output;
  }

  public void setOutput(String output)
  {
    this.output = output;
  }

  public void execute()
  {
    map = new HashMap();
    for (Iterator it = src.iterator(); it.hasNext();)
    {
      FileInputStream fis = null;
      try
      {
        fis = new FileInputStream((String)it.next());
        References refs = new References();
        refs.load(fis);
        for (Iterator it2 = refs.getPluginRefs().iterator(); it2.hasNext();)
        {
          PluginRef pluginRef = (PluginRef)it2.next();
          for (Iterator it3 = pluginRef.getClassRefs().iterator(); it3.hasNext();)
          {
            ClassRef classRef = (ClassRef)it3.next();
            String name = classRef.getName();
            int refCount = classRef.getRefCount();
            Integer refTotal = (Integer)map.get(name);
            if (refTotal == null)
            {
              refTotal = new Integer(0);
            }
            refTotal = new Integer(refCount + refTotal.intValue());
            map.put(name, refTotal);
          }
        }
      }
      catch (Throwable t)
      {
        throw new RuntimeException(t);
      }
      finally
      {
        if (fis != null)
        {
          try
          {
            fis.close();
          }
          catch (IOException ioe)
          {
          }
        }
      }
    }
    genCSV();
    genHTML();
  }

  public void genCSV()
  {
    FileWriter fw = null;
    try
    {
      fw = new FileWriter(output);
      for (Iterator it = map.keySet().iterator(); it.hasNext();)
      {
        String name = (String)it.next();
        fw.write(name);
        fw.write(",");
        fw.write(((Integer)map.get(name)).toString());
        fw.write("\n");
      }
    }
    catch (IOException ioe)
    {
      throw new RuntimeException(ioe);
    }
    finally
    {
      if (fw != null)
      {
        try
        {
          fw.close();
        }
        catch (IOException ioe)
        {
        }
      }
    }
  }

  public void genHTML()
  {
    try
    {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      baos.write("<root>".getBytes());
      for (Iterator it = map.keySet().iterator(); it.hasNext();)
      {
        String name = (String)it.next();
        baos.write("<class name=\"".getBytes());
        baos.write(name.getBytes());
        baos.write("\" ref=\"".getBytes());
        baos.write(((Integer)map.get(name)).toString().getBytes());
        baos.write("\"/>".getBytes());
      }
      baos.write("</root>".getBytes());
      baos.close();
      XSLUtil.transform
      (
        ClassLoader.getSystemResourceAsStream("org/eclipse/wtp/releng/tools/component/xsl/combine-class2ref.xsl"),
        new ByteArrayInputStream(baos.toByteArray()),
        new FileOutputStream(output + ".html")
      );
    }
    catch (Throwable t)
    {
      throw new RuntimeException(t);
    }
  }

  public static void main(String[] args)
  {
    CommandOptionParser optionParser = new CommandOptionParser(args);
    Map options = optionParser.getOptions();
    Collection src = (Collection)options.get("src");
    Collection output = (Collection)options.get("output");
    if (src == null || output == null || src.isEmpty() || output.isEmpty())
    {
      printUsage();
      System.exit(-1);
    }

    CombineClass2Reference class2Ref = new CombineClass2Reference();
    class2Ref.setSrc(src);
    class2Ref.setOutput((String)output.iterator().next());
    class2Ref.execute();
  }

  private static void printUsage()
  {
    System.out.println("Usage: java org.eclipse.wtp.releng.tools.component.adopters.CombineClass2Reference -src <src> -output <output>");
    System.out.println("");
    System.out.println("\t-src\t\t<src>\t\tlocation of your usage reports");
    System.out.println("\t-output\t<output>\t\tlocation of the output file");
  }
}